
# q-expansion of newform 5390.2.a.x, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5390
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [-1], [1], [0], [-1], [-2], [3], [1], [6], [-9], [-5], [5], [6], [8], [-6], [9], [-6], [-5], [8], [-9], [10], [14], [6], [15], [-8], [18], [16], [-12], [2], [-12], [-16], [-3], [12], [4], [21], [2], [-5], [5], [9], [-6], [24], [-14], [12], [-7], [12], [25], [-13], [10], [-18], [22], [-27], [-6], [-14], [18], [-18], [21], [12], [-20], [8], [18], [22], [-30], [-2], [3], [-14], [-9], [32], [-13], [18], [-2], [30], [24], [4], [14], [-10], [-30], [6], [-2], [3], [-32], [-12], [-28], [0], [16], [28], [12], [-6], [-1], [15], [14], [15], [0], [8], [-9], [32], [24], [-24], [-30], [28], [-13], [44], [-30], [-6], [-12], [-31], [-26], [-27], [18], [-3], [10], [-5], [26], [-36], [16], [-1], [-9], [-5], [6], [-9], [-27], [34], [41], [0], [-9], [10], [-3], [-34], [15], [-26], [4], [-16], [-9], [-7], [2], [-30], [-26], [-45], [-44], [-30], [18], [43], [18], [-40], [-48], [4], [24], [-38], [3], [-26], [-6], [-34], [-18], [-25], [-48], [-7], [3], [-52], [21], [-38], [-27], [3], [-21], [17], [-36], [-48], [42], [-40], [28], [14], [27], [-9], [58], [-30], [-7], [-56], [21], [44], [-12], [19], [55], [62], [-3], [-13], [15], [18], [-18], [-52], [28], [26], [12], [52], [24], [62], [-24], [12], [6], [-40], [14], [-42], [-45], [15], [-38], [37], [-32], [-60], [3], [-8], [-24], [-66], [-8], [-25], [0], [8], [69], [-48], [37], [5], [-6], [24], [27], [-22], [-14], [45], [20], [-24], [5], [-6], [51], [-32], [60], [-10], [43], [50], [-66], [-68], [-21], [-35], [42], [-72], [63], [27], [28], [-20], [2], [30], [30], [40], [42], [59], [45], [50], [-24], [6], [61], [-12], [-69], [-4], [46], [-42], [61], [56], [0], [7], [-26], [48], [-8], [-12], [63], [-76], [78], [46], [23], [-80], [2], [67], [34], [-48], [-10], [2], [63], [-39], [68], [-24], [-5], [-44], [21], [74], [54], [-59], [-42], [0], [-12], [-39], [-21], [-46], [-54], [64], [-51], [-60], [58], [1], [-48], [56], [-36], [-22], [56], [-60], [16], [-15], [32], [-36], [18], [-60], [53], [63], [34], [54], [-72], [-26], [-12], [-5], [-82], [-30], [-34], [3], [37], [2], [28], [3], [-84], [11], [30], [52], [66], [47], [-33], [38], [3], [67], [61], [-34], [-18], [21], [-73], [81], [9], [52], [-37], [-24], [-39], [-35], [-22], [12], [-53], [-19], [69], [63], [-69], [78], [-12], [50], [-24], [-12], [21], [-77], [-16], [-78], [11], [-16], [60], [64], [-48], [69], [-86], [74], [0], [-42], [-77], [36], [-23], [72], [-6], [44], [27], [-8], [-60], [80], [85], [20], [90], [-91], [81], [72], [-8], [24], [26], [-74], [57], [50], [-3], [85], [-6], [-76], [-72], [6], [-74], [-49], [-30], [-47], [-66], [-98], [0], [84], [-52], [-10], [0], [-96], [76], [18], [-99], [-24], [-56], [-72], [-39], [-74], [33], [-21], [0], [52], [42]]
